/*
 *  ofxMultiTouch.h
 *  openFrameworks
 *
 *  Created by Mehmet Akten on 15/01/2009.
 *  Copyright 2009 MSA Visuals Ltd.. All rights reserved.
 *
 */
#pragma once

#include <list>
#include "ofxMultiTouchListener.h"

class ofxMultiTouchHandler : public ofxMultiTouchListener {
public:
	void addListener(ofxMultiTouchListener* o) {
		listeners.push_back(o);
	}
	
	void removeListener(ofxMultiTouchListener* o) {
		listeners.remove(o);
	}
	
	// multitouch engine should call this when a 'down' is detected
	// use void *data to send any platform specific data
	void touchDown(int x, int y, int touchId, void *data = 0) {
		for(std::list<ofxMultiTouchListener*>::iterator it=listeners.begin(); it!=listeners.end(); ++it) {
			ofxMultiTouchListener* o = *it;
			o->touchDown(x, y, touchId, data);
		}
	}
	
	// multitouch engine should call this when a 'move' (drag) is detected
	// use void *data to send any platform specific data
	void touchMoved(int x, int y, int touchId, void *data = 0) {
		for(std::list<ofxMultiTouchListener*>::iterator it=listeners.begin(); it!=listeners.end(); ++it) {
			ofxMultiTouchListener* o = *it;
			o->touchMoved(x, y, touchId, data);
		}
	}
	
	// multitouch engine should call this when an 'up' is detected
	// use void *data to send any platform specific data
	void touchUp(int x, int y, int touchId, void *data = 0) {
		for(std::list<ofxMultiTouchListener*>::iterator it=listeners.begin(); it!=listeners.end(); ++it) {
			ofxMultiTouchListener* o = *it;
			o->touchUp(x, y, touchId, data);
		}
	}
	
	// multitouch engine should call this when a 'double tap' is detected
	// use void *data to send any platform specific data	
	void touchDoubleTap(int x, int y, int touchId, void *data = 0) {
		for(std::list<ofxMultiTouchListener*>::iterator it=listeners.begin(); it!=listeners.end(); ++it) {
			ofxMultiTouchListener* o = *it;
			o->touchDoubleTap(x, y, touchId, data);
		}
	}

	
protected:
	std::list<ofxMultiTouchListener*> listeners;
	
};

extern ofxMultiTouchHandler ofxMultiTouch;

